@@ -1,21 +1,21 @@
 from collections.abc import Iterable
-from typing import Any
 
 from sapiopylib.rest.DataMgmtService import DataMgmtServer
 from sapiopylib.rest.User import SapioUser
 from sapiopylib.rest.pojo.CustomReport import ReportColumn, CustomReport, CustomReportCriteria, RawReportTerm
-from sapiopylib.rest.pojo.webhook.WebhookContext import SapioWebhookContext
+
+from sapiopycommons.general.aliases import UserIdentifier, FieldValue, AliasUtil, FieldIdentifierKey
 
 
 # FR-46064 - Initial port of PyWebhookUtils to sapiopycommons.
 class CustomReportUtil:
     @staticmethod
-    def run_system_report(context: SapioWebhookContext | SapioUser,
+    def run_system_report(context: UserIdentifier,
                           report_name: str,
-                          filters: dict[str, Iterable[Any]] | None = None,
+                          filters: dict[FieldIdentifierKey, Iterable[FieldValue]] | None = None,
                           page_limit: int | None = None,
                           page_size: int | None = None,
-                          page_number: int | None = None) -> list[dict[str, Any]]:
+                          page_number: int | None = None) -> list[dict[str, FieldValue]]:
         """
         Run a system report and return the results of that report as a list of dictionaries for the values of each
         column in each row.
@@ -29,8 +29,10 @@ class CustomReportUtil:
             filter on. Only those headers that both the filters and the custom report share will take effect. That is,
             any filters that have a header name that isn't in the custom report will be ignored.
         :param page_limit: The maximum number of pages to query. If None, exhausts all possible pages.
-        :param page_size: The size of each page of results in the search. If None, the page size is set by the server.
-        :param page_number: The page number to start the search from, If None, starts on the first page.
+        :param page_size: The size of each page of results in the search. If None, the page size is set by the server,
+            which may be unlimited.
+        :param page_number: The page number to start the search from, If None, starts on the first page. Note that the
+            number of the first page is 0.
         :return: The results of the report listed row by row, mapping each cell to the header it is under. The header
             values in the dicts are the data field names of the columns.
             If two columns in the search have the same data field name but differing data type names, then the
@@ -38,19 +40,19 @@ class CustomReportUtil:
             had a Sample column with a data field name of Identifier and a Request column with the same data field name,
             then the dictionary keys for these columns would be Sample.Identifier and Request.Identifier respectively.
         """
-        results: tuple = CustomReportUtil.__exhaust_system_report(context, report_name, page_limit,
-                                                                  page_size, page_number)
+        results: tuple = CustomReportUtil._exhaust_system_report(context, report_name, page_limit,
+                                                                 page_size, page_number)
         columns: list[ReportColumn] = results[0]
-        rows: list[list[Any]] = results[1]
-        return CustomReportUtil.__process_results(rows, columns, filters)
+        rows: list[list[FieldValue]] = results[1]
+        return CustomReportUtil._process_results(rows, columns, filters)
 
     @staticmethod
-    def run_custom_report(context: SapioWebhookContext | SapioUser,
+    def run_custom_report(context: UserIdentifier,
                           report_criteria: CustomReportCriteria,
-                          filters: dict[str, Iterable[Any]] | None = None,
+                          filters: dict[FieldIdentifierKey, Iterable[FieldValue]] | None = None,
                           page_limit: int | None = None,
                           page_size: int | None = None,
-                          page_number: int | None = None) -> list[dict[str, Any]]:
+                          page_number: int | None = None) -> list[dict[str, FieldValue]]:
         """
         Run a custom report and return the results of that report as a list of dictionaries for the values of each
         column in each row.
@@ -71,7 +73,8 @@ class CustomReportUtil:
         :param page_size: The size of each page of results in the search. If None, uses the value from the given report
             criteria. If not None, overwrites the value from the given report criteria.
         :param page_number: The page number to start the search from, If None, uses the value from the given report
-            criteria. If not None, overwrites the value from the given report criteria.
+            criteria. If not None, overwrites the value from the given report criteria. Note that the number of the
+            first page is 0.
         :return: The results of the report listed row by row, mapping each cell to the header it is under. The header
             values in the dicts are the data field names of the columns.
             If two columns in the search have the same data field name but differing data type names, then the
@@ -79,19 +82,19 @@ class CustomReportUtil:
             had a Sample column with a data field name of Identifier and a Request column with the same data field name,
             then the dictionary keys for these columns would be Sample.Identifier and Request.Identifier respectively.
         """
-        results: tuple = CustomReportUtil.__exhaust_custom_report(context, report_criteria, page_limit,
-                                                                  page_size, page_number)
+        results: tuple = CustomReportUtil._exhaust_custom_report(context, report_criteria, page_limit,
+                                                                 page_size, page_number)
         columns: list[ReportColumn] = results[0]
-        rows: list[list[Any]] = results[1]
-        return CustomReportUtil.__process_results(rows, columns, filters)
+        rows: list[list[FieldValue]] = results[1]
+        return CustomReportUtil._process_results(rows, columns, filters)
 
     @staticmethod
-    def run_quick_report(context: SapioWebhookContext | SapioUser,
+    def run_quick_report(context: UserIdentifier,
                          report_term: RawReportTerm,
-                         filters: dict[str, Iterable[Any]] | None = None,
+                         filters: dict[FieldIdentifierKey, Iterable[FieldValue]] | None = None,
                          page_limit: int | None = None,
                          page_size: int | None = None,
-                         page_number: int | None = None) -> list[dict[str, Any]]:
+                         page_number: int | None = None) -> list[dict[str, FieldValue]]:
         """
         Run a quick report and return the results of that report as a list of dictionaries for the values of each
         column in each row.
@@ -107,19 +110,21 @@ class CustomReportUtil:
             filter on. Only those headers that both the filters and the custom report share will take effect. That is,
             any filters that have a header name that isn't in the custom report will be ignored.
         :param page_limit: The maximum number of pages to query. If None, exhausts all possible pages.
-        :param page_size: The size of each page of results in the search. If None, the page size is set by the server.
-        :param page_number: The page number to start the search from, If None, starts on the first page.
+        :param page_size: The size of each page of results in the search. If None, the page size is set by the server,
+            which may be unlimited.
+        :param page_number: The page number to start the search from, If None, starts on the first page. Note that the
+            number of the first page is 0.
         :return: The results of the report listed row by row, mapping each cell to the header it is under. The header
             values in the dicts are the data field names of the columns.
         """
-        results: tuple = CustomReportUtil.__exhaust_quick_report(context, report_term, page_limit,
-                                                                 page_size, page_number)
+        results: tuple = CustomReportUtil._exhaust_quick_report(context, report_term, page_limit,
+                                                                page_size, page_number)
         columns: list[ReportColumn] = results[0]
-        rows: list[list[Any]] = results[1]
-        return CustomReportUtil.__process_results(rows, columns, filters)
+        rows: list[list[FieldValue]] = results[1]
+        return CustomReportUtil._process_results(rows, columns, filters)
 
     @staticmethod
-    def get_system_report_criteria(context: SapioWebhookContext | SapioUser, report_name: str) -> CustomReport:
+    def get_system_report_criteria(context: UserIdentifier, report_name: str) -> CustomReport:
         """
         Retrieve a custom report from the system given the name of the report. This works by querying the system report
         with a page number and size of 1 to minimize the amount of data transfer needed to retrieve the report's config.
@@ -134,98 +139,113 @@ class CustomReportUtil:
         :param report_name: The name of the system report to run.
         :return: The CustomReport object for the given system report name.
         """
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         report_man = DataMgmtServer.get_custom_report_manager(user)
-        return report_man.run_system_report_by_name(report_name, 1, 1)
+        return report_man.run_system_report_by_name(report_name, 1, 0)
 
     @staticmethod
-    def __exhaust_system_report(context: SapioWebhookContext | SapioUser,
-                                report_name: str,
-                                page_limit: int | None,
-                                page_size: int | None,
-                                page_number: int | None) \
-            -> tuple[list[ReportColumn], list[list[Any]]]:
+    def _exhaust_system_report(context: UserIdentifier,
+                               report_name: str,
+                               page_limit: int | None,
+                               page_size: int | None,
+                               page_number: int | None) \
+            -> tuple[list[ReportColumn], list[list[FieldValue]]]:
         """
         Given a system report, iterate over every page of the report and collect the results
         until there are no remaining pages.
         """
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         report_man = DataMgmtServer.get_custom_report_manager(user)
 
+        # If a page size was provided but no page number was provided, then set the page number to 0,
+        # as both parameters are necessary in order to get paged results.
+        if page_size is not None and page_number is None:
+            page_number = 0
+
         result = None
         has_next_page: bool = True
-        rows: list[list[Any]] = []
+        rows: list[list[FieldValue]] = []
         cur_page: int = 1
         while has_next_page and (not page_limit or cur_page <= page_limit):
             result = report_man.run_system_report_by_name(report_name, page_size, page_number)
             page_size = result.page_size
-            page_number = result.page_number
+            page_number = result.page_number + 1
             has_next_page = result.has_next_page
             rows.extend(result.result_table)
             cur_page += 1
         return result.column_list, rows
 
     @staticmethod
-    def __exhaust_custom_report(context: SapioWebhookContext | SapioUser,
-                                report: CustomReportCriteria,
-                                page_limit: int | None,
-                                page_size: int | None,
-                                page_number: int | None) \
-            -> tuple[list[ReportColumn], list[list[Any]]]:
+    def _exhaust_custom_report(context: UserIdentifier,
+                               report: CustomReportCriteria,
+                               page_limit: int | None,
+                               page_size: int | None,
+                               page_number: int | None) \
+            -> tuple[list[ReportColumn], list[list[FieldValue]]]:
         """
         Given a custom report, iterate over every page of the report and collect the results
         until there are no remaining pages.
         """
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         report_man = DataMgmtServer.get_custom_report_manager(user)
 
+        # If a page size was provided but no page number was provided, then set the page number to 0,
+        # as both parameters are necessary in order to get paged results.
+        if page_size is not None and page_number is None:
+            page_number = 0
+
         result = None
         if page_size is not None:
             report.page_size = page_size
         if page_number is not None:
             report.page_number = page_number
         has_next_page: bool = True
-        rows: list[list[Any]] = []
+        rows: list[list[FieldValue]] = []
         cur_page: int = 1
         while has_next_page and (not page_limit or cur_page <= page_limit):
             result = report_man.run_custom_report(report)
             report.page_size = result.page_size
-            report.page_number = result.page_number
+            report.page_number = result.page_number + 1
             has_next_page = result.has_next_page
             rows.extend(result.result_table)
             cur_page += 1
         return result.column_list, rows
 
     @staticmethod
-    def __exhaust_quick_report(context: SapioWebhookContext | SapioUser,
-                               report_term: RawReportTerm,
-                               page_limit: int | None,
-                               page_size: int | None,
-                               page_number: int | None) \
-            -> tuple[list[ReportColumn], list[list[Any]]]:
+    def _exhaust_quick_report(context: UserIdentifier,
+                              report_term: RawReportTerm,
+                              page_limit: int | None,
+                              page_size: int | None,
+                              page_number: int | None) \
+            -> tuple[list[ReportColumn], list[list[FieldValue]]]:
         """
         Given a quick report, iterate over every page of the report and collect the results
         until there are no remaining pages.
         """
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         report_man = DataMgmtServer.get_custom_report_manager(user)
 
+        # If a page size was provided but no page number was provided, then set the page number to 0,
+        # as both parameters are necessary in order to get paged results.
+        if page_size is not None and page_number is None:
+            page_number = 0
+
         result = None
         has_next_page: bool = True
-        rows: list[list[Any]] = []
+        rows: list[list[FieldValue]] = []
         cur_page: int = 1
         while has_next_page and (not page_limit or cur_page <= page_limit):
             result = report_man.run_quick_report(report_term, page_size, page_number)
             page_size = result.page_size
-            page_number = result.page_number
+            page_number = result.page_number + 1
             has_next_page = result.has_next_page
             rows.extend(result.result_table)
             cur_page += 1
         return result.column_list, rows
 
     @staticmethod
-    def __process_results(rows: list[list[Any]], columns: list[ReportColumn],
-                          filters: dict[str, Iterable[Any]] | None) -> list[dict[str, Any]]:
+    def _process_results(rows: list[list[FieldValue]], columns: list[ReportColumn],
+                         filters: dict[FieldIdentifierKey, Iterable[FieldValue]] | None) -> list[dict[str, FieldValue]]:
         """
         Given the results of a report as a list of row values and the report's columns, combine these lists to
         result in a singular list of dictionaries for each row in the results.
@@ -243,9 +263,12 @@ class CustomReportUtil:
             else:
                 encountered_names.append(field_name)
 
-        ret: list[dict[str, Any]] = []
+        if filters:
+            filters: dict[str, Iterable[FieldValue]] = AliasUtil.to_data_field_names_dict(filters)
+
+        ret: list[dict[str, FieldValue]] = []
         for row in rows:
-            row_data: dict[str, Any] = {}
+            row_data: dict[str, FieldValue] = {}
             filter_row: bool = False
             for value, column in zip(row, columns):
                 header: str = column.data_field_name
