@@ -1,10 +1,15 @@
+from __future__ import annotations
+
+from weakref import WeakValueDictionary
+
+from sapiopylib.rest.User import SapioUser
 from sapiopylib.rest.pojo.DataRecord import DataRecord
-from sapiopylib.rest.pojo.webhook.VeloxRules import VeloxRuleType, VeloxRuleParser
+from sapiopylib.rest.pojo.eln.SapioELNEnums import ElnBaseDataType
 from sapiopylib.rest.pojo.webhook.WebhookContext import SapioWebhookContext
 from sapiopylib.rest.utils.recordmodel.RecordModelManager import RecordModelManager, RecordModelInstanceManager
 from sapiopylib.rest.utils.recordmodel.RecordModelWrapper import WrappedType
 
-from sapiopycommons.general.aliases import FieldMap
+from sapiopycommons.general.aliases import FieldMap, DataTypeIdentifier, AliasUtil
 from sapiopycommons.general.exceptions import SapioException
 
 
@@ -12,7 +17,6 @@ from sapiopycommons.general.exceptions import SapioException
 class OnSaveRuleHandler:
     """
     A class which helps with the parsing and navigation of the on save rule result map of a webhook context.
-    TODO: Add functionality around the VeloxRuleType of the rule results.
     """
     __context: SapioWebhookContext
     """The context that this handler is working from."""
@@ -34,7 +38,25 @@ class OnSaveRuleHandler:
     """A mapping of record IDs of records in the context.data_record_list to the field maps related to that
     record, each grouping of field maps being mapped by its data type."""
 
+    __instances: WeakValueDictionary[SapioUser, OnSaveRuleHandler] = WeakValueDictionary()
+    __initialized: bool
+
+    def __new__(cls, context: SapioWebhookContext):
+        if context.velox_on_save_result_map is None:
+            raise SapioException("No Velox on save rule result map in context for OnSaveRuleHandler to parse.")
+        user = context if isinstance(context, SapioUser) else context.user
+        obj = cls.__instances.get(user)
+        if not obj:
+            obj = object.__new__(cls)
+            obj.__initialized = False
+            cls.__instances[user] = obj
+        return obj
+
     def __init__(self, context: SapioWebhookContext):
+        if self.__initialized:
+            return
+        self.__initialized = True
+
         if context.velox_on_save_result_map is None:
             raise SapioException("No Velox on save rule result map in context for OnSaveRuleHandler to parse.")
         self.__context = context
@@ -51,9 +73,6 @@ class OnSaveRuleHandler:
         self.__base_id_to_records = {}
         # Each record ID in the context has a list of results for that record.
         for record_id, rule_results in self.__context.velox_on_save_result_map.items():
-            # TODO: Record IDs are currently being stored in the map as strings instead of ints. This can be removed
-            #  once sapiopylib is fixed.
-            record_id = int(record_id)
             # Keep track of the records for this specific record ID.
             id_dict: dict[str, set[DataRecord]] = {}
             # The list of results for a record consist of a list of data records and a VeloxType that specifies
@@ -64,13 +83,8 @@ class OnSaveRuleHandler:
                     # Get the data type of this record. If this is an ELN type, ignore the digits.
                     data_type: str = record.data_type_name
                     # PR-46331: Ensure that all ELN types are converted to their base data type name.
-                    # TODO: Use ElnBaseDataType.is_eln_type when it is no longer bugged in sapiopylib.
-                    if data_type.startswith("ELNExperiment_"):
-                        data_type = "ELNExperiment"
-                    elif data_type.startswith("ELNExperimentDetail_"):
-                        data_type = "ELNExperimentDetail"
-                    elif data_type.startswith("ELNSampleDetail_"):
-                        data_type = "ELNSampleDetail"
+                    if ElnBaseDataType.is_eln_type(data_type):
+                        data_type = ElnBaseDataType.get_base_type(data_type).data_type_name
                     # Update the list of records of this type that exist so far globally.
                     self.__records.setdefault(data_type, set()).add(record)
                     # Do the same for the list of records of this type that relate to this record ID.
@@ -82,24 +96,11 @@ class OnSaveRuleHandler:
         self.__base_id_to_field_maps = {}
         # Repeat the same thing for the field map results.
         for record_id, rule_results in self.__context.velox_on_save_field_map_result_map.items():
-            # TODO: Record IDs are currently being stored in the map as strings instead of ints. This can be removed
-            #  once sapiopylib is fixed.
-            record_id = int(record_id)
             id_dict: dict[str, dict[int, FieldMap]] = {}
             for record_result in rule_results:
-                # TODO: sapiopylib currently has a bug where this velox_type_pojo variable is stored as a dict instead
-                #  of as the intended VeloxRuleType object. Parse that dict as a VeloxRuleType before use.
-                velox_type: VeloxRuleType | dict = record_result.velox_type_pojo
-                if isinstance(velox_type, dict):
-                    velox_type: VeloxRuleType = VeloxRuleParser.parse_velox_rule_type(velox_type)
-                data_type: str = velox_type.data_type_name
-                # TODO: Use ElnBaseDataType.is_eln_type when it is no longer bugged in sapiopylib.
-                if data_type.startswith("ELNExperiment_"):
-                    data_type = "ELNExperiment"
-                elif data_type.startswith("ELNExperimentDetail_"):
-                    data_type = "ELNExperimentDetail"
-                elif data_type.startswith("ELNSampleDetail_"):
-                    data_type = "ELNSampleDetail"
+                data_type: str = record_result.velox_type_pojo.data_type_name
+                if ElnBaseDataType.is_eln_type(data_type):
+                    data_type = ElnBaseDataType.get_base_type(data_type).data_type_name
                 for field_map in record_result.field_map_list:
                     rec_id: int = field_map.get("RecordId")
                     self.__field_maps.setdefault(data_type, {}).update({rec_id: field_map})
@@ -120,7 +121,7 @@ class OnSaveRuleHandler:
         """
         return list(self.__base_id_to_field_maps.keys())
 
-    def get_records(self, data_type: str, record_id: int | None = None) -> list[DataRecord]:
+    def get_records(self, data_type: DataTypeIdentifier, record_id: int | None = None) -> list[DataRecord]:
         """
         Get records from the cached context with the given data type. Capable of being filtered to searching within
         the context of a record ID. If the given data type or record ID does not exist in the context,
@@ -131,11 +132,12 @@ class OnSaveRuleHandler:
             data type from every ID. If an ID is provided, but it does not exist in the context, returns an empty list.
         :return: The records from the context that match the input parameters.
         """
+        data_type: str = AliasUtil.to_data_type_name(data_type)
         records: dict[str, set[DataRecord]] = self.__base_id_to_records.get(record_id, {}) if record_id else self.__records
         return list(records.get(data_type, []))
 
     # FR-46701: Add functions to the rule handlers for accessing the field maps of inaccessible records in the context.
-    def get_field_maps(self, data_type: str, record_id: int | None = None) -> list[FieldMap]:
+    def get_field_maps(self, data_type: DataTypeIdentifier, record_id: int | None = None) -> list[FieldMap]:
         """
         Get field maps from the cached context with the given data type. Capable of being filtered to searching within
         the context of a record ID. If the given data type or record ID does not exist in the context,
@@ -151,6 +153,7 @@ class OnSaveRuleHandler:
             list.
         :return: The field maps from the context that match the input parameters.
         """
+        data_type: str = AliasUtil.to_data_type_name(data_type)
         field_maps: dict[str, dict[int, FieldMap]] = self.__base_id_to_field_maps.get(record_id, {}) if record_id else self.__field_maps
         return list(field_maps.get(data_type, {}).values())
 
